@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 2018-2023. All Rights Reserved.
+ * Copyright Ericsson AB 2018-2024. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -41,18 +41,27 @@
 #endif
 #include <windows.h>
 #include <Ws2tcpip.h>   /* NEED VC 6.0 or higher */
+
 /* Visual studio 2008+: NTDDI_VERSION needs to be set for iphlpapi.h
- * to define the right structures. It needs to be set to WINXP (or LONGHORN)
- * for IPV6 to work and it's set lower by default, so we need to change it.
+ * to define the right structures.
+ * It needs to be set higher for IPV6 to work and it's set lower by default,
+ * so we need to change it.
  */
 #ifdef HAVE_SDKDDKVER_H
 #  include <sdkddkver.h>
 #  ifdef NTDDI_VERSION
 #    undef NTDDI_VERSION
 #  endif
-#  define NTDDI_VERSION NTDDI_WINXP
+#  define NTDDI_VERSION NTDDI_WIN10_RS2
 #endif
 #include <iphlpapi.h>
+#include <mstcpip.h>
+
+/* Since we can't get configure to work properly on Windows... */
+#if defined(AF_UNIX)
+#include <afunix.h>
+#define HAVE_STRUCT_SOCKADDR_UN_SUN_PATH 1
+#endif
 
 #else /* !__WIN32__ */
 
@@ -232,13 +241,22 @@ typedef long ssize_t;
     GLOBAL_ATOM_DEF(bindtodevice);             \
     GLOBAL_ATOM_DEF(block_source);             \
     GLOBAL_ATOM_DEF(broadcast);                \
+    GLOBAL_ATOM_DEF(bsp_state);                \
     GLOBAL_ATOM_DEF(busy_poll);                \
+    GLOBAL_ATOM_DEF(bytes_in);                 \
+    GLOBAL_ATOM_DEF(bytes_in_flight);          \
+    GLOBAL_ATOM_DEF(bytes_out);                \
+    GLOBAL_ATOM_DEF(bytes_reordered);          \
+    GLOBAL_ATOM_DEF(bytes_retrans);            \
     GLOBAL_ATOM_DEF(cancel);                   \
+    GLOBAL_ATOM_DEF(cancelled);                \
     GLOBAL_ATOM_DEF(cantconfig);	       \
     GLOBAL_ATOM_DEF(chaos);                    \
     GLOBAL_ATOM_DEF(checksum);                 \
     GLOBAL_ATOM_DEF(close);                    \
     GLOBAL_ATOM_DEF(closed);                   \
+    GLOBAL_ATOM_DEF(close_wait);               \
+    GLOBAL_ATOM_DEF(closing);                  \
     GLOBAL_ATOM_DEF(cmsg_cloexec);             \
     GLOBAL_ATOM_DEF(command);                  \
     GLOBAL_ATOM_DEF(completion);               \
@@ -248,12 +266,14 @@ typedef long ssize_t;
     GLOBAL_ATOM_DEF(connect);                  \
     GLOBAL_ATOM_DEF(connected);                \
     GLOBAL_ATOM_DEF(connecting);               \
+    GLOBAL_ATOM_DEF(connection_time);          \
     GLOBAL_ATOM_DEF(context);                  \
     GLOBAL_ATOM_DEF(cork);                     \
     GLOBAL_ATOM_DEF(counters);                 \
     GLOBAL_ATOM_DEF(credentials);              \
     GLOBAL_ATOM_DEF(ctrl);                     \
     GLOBAL_ATOM_DEF(ctrunc);                   \
+    GLOBAL_ATOM_DEF(cwnd);                     \
     GLOBAL_ATOM_DEF(data);                     \
     GLOBAL_ATOM_DEF(data_size);                \
     GLOBAL_ATOM_DEF(debug);                    \
@@ -261,6 +281,7 @@ typedef long ssize_t;
     GLOBAL_ATOM_DEF(default_send_params);      \
     GLOBAL_ATOM_DEF(delayed_ack_time);         \
     GLOBAL_ATOM_DEF(dgram);                    \
+    GLOBAL_ATOM_DEF(dhcprunning);              \
     GLOBAL_ATOM_DEF(disable_fragments);        \
     GLOBAL_ATOM_DEF(dlci);                     \
     GLOBAL_ATOM_DEF(dma);                      \
@@ -272,6 +293,7 @@ typedef long ssize_t;
     GLOBAL_ATOM_DEF(drop_source_membership);   \
     GLOBAL_ATOM_DEF(dstopts);                  \
     GLOBAL_ATOM_DEF(dup);		       \
+    GLOBAL_ATOM_DEF(dup_acks_in);              \
     GLOBAL_ATOM_DEF(dying);		       \
     GLOBAL_ATOM_DEF(dynamic);                  \
     GLOBAL_ATOM_DEF(echo);                     \
@@ -284,14 +306,19 @@ typedef long ssize_t;
     GLOBAL_ATOM_DEF(errqueue);                 \
     GLOBAL_ATOM_DEF(esp_network_level);        \
     GLOBAL_ATOM_DEF(esp_trans_level);          \
+    GLOBAL_ATOM_DEF(established);              \
     GLOBAL_ATOM_DEF(ether);                    \
     GLOBAL_ATOM_DEF(eui64);                    \
     GLOBAL_ATOM_DEF(events);                   \
+    GLOBAL_ATOM_DEF(exclusiveaddruse);         \
     GLOBAL_ATOM_DEF(explicit_eor);             \
     GLOBAL_ATOM_DEF(faith);                    \
     GLOBAL_ATOM_DEF(false);                    \
     GLOBAL_ATOM_DEF(family);                   \
     GLOBAL_ATOM_DEF(fastroute);                \
+    GLOBAL_ATOM_DEF(fast_retrans);             \
+    GLOBAL_ATOM_DEF(fin_wait_1);               \
+    GLOBAL_ATOM_DEF(fin_wait_2);               \
     GLOBAL_ATOM_DEF(flags);                    \
     GLOBAL_ATOM_DEF(flowinfo);                 \
     GLOBAL_ATOM_DEF(fragment_interleave);      \
@@ -305,6 +332,7 @@ typedef long ssize_t;
     GLOBAL_ATOM_DEF(hoplimit);                 \
     GLOBAL_ATOM_DEF(hopopts);                  \
     GLOBAL_ATOM_DEF(host);                     \
+    GLOBAL_ATOM_DEF(hwaddr);                   \
     GLOBAL_ATOM_DEF(icmp);                     \
     GLOBAL_ATOM_DEF(icmp6);                    \
     GLOBAL_ATOM_DEF(ieee802);                  \
@@ -324,6 +352,7 @@ typedef long ssize_t;
     GLOBAL_ATOM_DEF(ip);                       \
     GLOBAL_ATOM_DEF(ipcomp_level);             \
     GLOBAL_ATOM_DEF(ipip);                     \
+    GLOBAL_ATOM_DEF(iplevel);                  \
     GLOBAL_ATOM_DEF(ipv6);                     \
     GLOBAL_ATOM_DEF(irq);                      \
     GLOBAL_ATOM_DEF(i_want_mapped_v4_addr);    \
@@ -334,6 +363,7 @@ typedef long ssize_t;
     GLOBAL_ATOM_DEF(keepintvl);                \
     GLOBAL_ATOM_DEF(kernel);                   \
     GLOBAL_ATOM_DEF(knowsepoch);	       \
+    GLOBAL_ATOM_DEF(last_ack);                 \
     GLOBAL_ATOM_DEF(leave_group);              \
     GLOBAL_ATOM_DEF(level);                    \
     GLOBAL_ATOM_DEF(linger);                   \
@@ -341,6 +371,7 @@ typedef long ssize_t;
     GLOBAL_ATOM_DEF(link0);                    \
     GLOBAL_ATOM_DEF(link1);                    \
     GLOBAL_ATOM_DEF(link2);                    \
+    GLOBAL_ATOM_DEF(listen);                   \
     GLOBAL_ATOM_DEF(local);                    \
     GLOBAL_ATOM_DEF(localtlk);                 \
     GLOBAL_ATOM_DEF(local_auth_chunks);        \
@@ -349,17 +380,22 @@ typedef long ssize_t;
     GLOBAL_ATOM_DEF(lower_up);                 \
     GLOBAL_ATOM_DEF(mark);                     \
     GLOBAL_ATOM_DEF(master);                   \
+    GLOBAL_ATOM_DEF(max);                      \
     GLOBAL_ATOM_DEF(maxburst);                 \
+    GLOBAL_ATOM_DEF(maxdg);                    \
     GLOBAL_ATOM_DEF(maxseg);                   \
+    GLOBAL_ATOM_DEF(max_msg_size);             \
     GLOBAL_ATOM_DEF(md5sig);                   \
     GLOBAL_ATOM_DEF(mem_end);                  \
     GLOBAL_ATOM_DEF(mem_start);                \
     GLOBAL_ATOM_DEF(metricom);                 \
     GLOBAL_ATOM_DEF(mincost);                  \
     GLOBAL_ATOM_DEF(minttl);                   \
+    GLOBAL_ATOM_DEF(min_rtt);                  \
     GLOBAL_ATOM_DEF(monitor);		       \
     GLOBAL_ATOM_DEF(more);                     \
     GLOBAL_ATOM_DEF(msfilter);                 \
+    GLOBAL_ATOM_DEF(mss);                      \
     GLOBAL_ATOM_DEF(mtu);                      \
     GLOBAL_ATOM_DEF(mtu_discover);             \
     GLOBAL_ATOM_DEF(multicast);                \
@@ -392,7 +428,9 @@ typedef long ssize_t;
     GLOBAL_ATOM_DEF(num_unexpected_writes);    \
     GLOBAL_ATOM_DEF(num_unknown_cmds);         \
     GLOBAL_ATOM_DEF(oactive);		       \
+    GLOBAL_ATOM_DEF(off);                      \
     GLOBAL_ATOM_DEF(ok);                       \
+    GLOBAL_ATOM_DEF(on);                       \
     GLOBAL_ATOM_DEF(oob);                      \
     GLOBAL_ATOM_DEF(oobinline);                \
     GLOBAL_ATOM_DEF(options);                  \
@@ -417,17 +455,21 @@ typedef long ssize_t;
     GLOBAL_ATOM_DEF(portrange);                \
     GLOBAL_ATOM_DEF(portsel);                  \
     GLOBAL_ATOM_DEF(primary_addr);             \
-    GLOBAL_ATOM_DEF(priority);                 \
     GLOBAL_ATOM_DEF(prim_file);                \
+    GLOBAL_ATOM_DEF(priority);                 \
+    GLOBAL_ATOM_DEF(private);                  \
     GLOBAL_ATOM_DEF(promisc);                  \
     GLOBAL_ATOM_DEF(pronet);                   \
     GLOBAL_ATOM_DEF(protocol);                 \
     GLOBAL_ATOM_DEF(pup);                      \
     GLOBAL_ATOM_DEF(raw);                      \
+    GLOBAL_ATOM_DEF(rawip);                    \
     GLOBAL_ATOM_DEF(rcvbuf);                   \
     GLOBAL_ATOM_DEF(rcvbufforce);              \
     GLOBAL_ATOM_DEF(rcvlowat);                 \
     GLOBAL_ATOM_DEF(rcvtimeo);                 \
+    GLOBAL_ATOM_DEF(rcv_buf);                  \
+    GLOBAL_ATOM_DEF(rcv_wnd);                  \
     GLOBAL_ATOM_DEF(rdm);                      \
     GLOBAL_ATOM_DEF(read_byte);                \
     GLOBAL_ATOM_DEF(read_fails);               \
@@ -454,9 +496,11 @@ typedef long ssize_t;
     GLOBAL_ATOM_DEF(reuseaddr);                \
     GLOBAL_ATOM_DEF(reuseport);                \
     GLOBAL_ATOM_DEF(rights);                   \
+    GLOBAL_ATOM_DEF(rm);                       \
     GLOBAL_ATOM_DEF(router_alert);             \
     GLOBAL_ATOM_DEF(rthdr);                    \
     GLOBAL_ATOM_DEF(rtoinfo);                  \
+    GLOBAL_ATOM_DEF(rtt);                      \
     GLOBAL_ATOM_DEF(running);                  \
     GLOBAL_ATOM_DEF(rxq_ovfl);                 \
     GLOBAL_ATOM_DEF(scope_id);                 \
@@ -478,6 +522,7 @@ typedef long ssize_t;
     GLOBAL_ATOM_DEF(sendmsg);                  \
     GLOBAL_ATOM_DEF(sendsrcaddr);              \
     GLOBAL_ATOM_DEF(sendto);                   \
+    GLOBAL_ATOM_DEF(sendv);                    \
     GLOBAL_ATOM_DEF(seqpacket);                \
     GLOBAL_ATOM_DEF(setfib);                   \
     GLOBAL_ATOM_DEF(set_peer_primary_addr);    \
@@ -488,15 +533,20 @@ typedef long ssize_t;
     GLOBAL_ATOM_DEF(sndbufforce);              \
     GLOBAL_ATOM_DEF(sndlowat);                 \
     GLOBAL_ATOM_DEF(sndtimeo);                 \
+    GLOBAL_ATOM_DEF(snd_wnd);                  \
     GLOBAL_ATOM_DEF(sockaddr);                 \
     GLOBAL_ATOM_DEF(socket);                   \
     GLOBAL_ATOM_DEF(socket_tag);               \
+    GLOBAL_ATOM_DEF(socktype);                 \
     GLOBAL_ATOM_DEF(spec_dst);                 \
     GLOBAL_ATOM_DEF(state);                    \
     GLOBAL_ATOM_DEF(status);                   \
     GLOBAL_ATOM_DEF(staticarp);		       \
     GLOBAL_ATOM_DEF(stream);                   \
     GLOBAL_ATOM_DEF(syncnt);                   \
+    GLOBAL_ATOM_DEF(syn_rcvd);                 \
+    GLOBAL_ATOM_DEF(syn_retrans);              \
+    GLOBAL_ATOM_DEF(syn_sent);                 \
     GLOBAL_ATOM_DEF(tclass);                   \
     GLOBAL_ATOM_DEF(tcp);                      \
     GLOBAL_ATOM_DEF(throughput);               \
@@ -504,6 +554,9 @@ typedef long ssize_t;
     GLOBAL_ATOM_DEF(tos);                      \
     GLOBAL_ATOM_DEF(transparent);              \
     GLOBAL_ATOM_DEF(timeout);                  \
+    GLOBAL_ATOM_DEF(timeout_episodes);         \
+    GLOBAL_ATOM_DEF(timestamp_enabled);        \
+    GLOBAL_ATOM_DEF(time_wait);                \
     GLOBAL_ATOM_DEF(true);                     \
     GLOBAL_ATOM_DEF(trunc);                    \
     GLOBAL_ATOM_DEF(ttl);                      \
@@ -604,26 +657,32 @@ GLOBAL_ERROR_REASON_ATOM_DEFS;
 
 #define COMPARE(A, B)        enif_compare((A), (B))
 #define COMPARE_PIDS(P1, P2) enif_compare_pids((P1), (P2))
-#define IS_ZERO(R)           (COMPARE((R), esock_atom_zero) == 0)
+#define IS_IDENTICAL(A, B)   enif_is_identical((A), (B))
+#define IS_ZERO(T)           (COMPARE((T), esock_atom_zero) == 0)
+#define IS_UNDEFINED(T)      IS_IDENTICAL((T), esock_atom_undefined)
+#define IS_OK(T)             IS_IDENTICAL((T), esock_atom_ok)
 
-#define IS_ATOM(E,    TE) enif_is_atom((E),   (TE))
-#define IS_BIN(E,     TE) enif_is_binary((E), (TE))
-#define IS_LIST(E,    TE) enif_is_list((E),   (TE))
-#define IS_MAP(E,     TE) enif_is_map((E), (TE))
-#define IS_NUM(E,     TE) enif_is_number((E), (TE))
-#define IS_TUPLE(E,   TE) enif_is_tuple((E),  (TE))
-#define IS_INTEGER(E, TE) esock_is_integer((E),   (TE))
+#define IS_ATOM(E,    TE)    enif_is_atom((E),   (TE))
+#define IS_BIN(E,     TE)    enif_is_binary((E), (TE))
+#define IS_LIST(E,    TE)    enif_is_list((E),   (TE))
+#define IS_MAP(E,     TE)    enif_is_map((E), (TE))
+#define IS_NUM(E,     TE)    enif_is_number((E), (TE))
+#define IS_TUPLE(E,   TE)    enif_is_tuple((E),  (TE))
+#define IS_INTEGER(E, TE)    esock_is_integer((E),   (TE))
 
-#define IS_PID_UNDEF(P)   enif_is_pid_undefined((P))
-#define SET_PID_UNDEF(P)  enif_set_pid_undefined((P))
+#define IS_PID_UNDEF(P)      enif_is_pid_undefined((P))
+#define SET_PID_UNDEF(P)     enif_set_pid_undefined((P))
 
 #define GET_ATOM_LEN(E, TE, LP) \
     enif_get_atom_length((E), (TE), (LP), ERL_NIF_LATIN1)
 #define GET_ATOM(E, TE, BP, MAX) \
     enif_get_atom((E), (TE), (BP), (MAX), ERL_NIF_LATIN1)
-#define GET_BIN(E, TE, BP)          enif_inspect_iolist_as_binary((E), (TE), (BP))
+#define GET_BIN(E, TE, BP)          \
+    enif_inspect_iolist_as_binary((E), (TE), (BP))
 #define GET_INT(E, TE, IP)          enif_get_int((E), (TE), (IP))
 #define GET_INT64(E, TE, IP)        enif_get_int64((E), (TE), (IP))
+#define GET_IOV(ME, EIOV, T, IOV)                       \
+    enif_inspect_iovec(NULL, (ME), (EIOV), (T), (IOV))
 #define GET_LIST_ELEM(E, L, HP, TP) enif_get_list_cell((E), (L), (HP), (TP))
 #define GET_LIST_LEN(E, L, LP)      enif_get_list_length((E), (L), (LP))
 #define GET_LONG(E, TE, LP)         enif_get_long((E), (TE), (LP))
@@ -637,7 +696,7 @@ GLOBAL_ERROR_REASON_ATOM_DEFS;
 #define GET_MAP_VAL(E, M, K, V)     enif_get_map_value((E), (M), (K), (V))
 
 #define ALLOC_BIN(SZ, BP)         enif_alloc_binary((SZ), (BP))
-#define REALLOC_BIN(SZ, BP)       enif_realloc_binary((SZ), (BP))
+#define REALLOC_BIN(BP, SZ)       enif_realloc_binary((BP), (SZ))
 #define FREE_BIN(BP)              enif_release_binary((BP))
 
 #define FREE_IOVEC(IV)            enif_free_iovec((IV))
