@@ -1,18 +1,19 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2010-2015. All Rights Reserved.
+%% Copyright Ericsson AB 2010-2024. All Rights Reserved.
 %%
-%% The contents of this file are subject to the Erlang Public License,
-%% Version 1.1, (the "License"); you may not use this file except in
-%% compliance with the License. You should have received a copy of the
-%% Erlang Public License along with this software. If not, it can be
-%% retrieved online at http://www.erlang.org/.
+%% Licensed under the Apache License, Version 2.0 (the "License");
+%% you may not use this file except in compliance with the License.
+%% You may obtain a copy of the License at
 %%
-%% Software distributed under the License is distributed on an "AS IS"
-%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
-%% the License for the specific language governing rights and limitations
-%% under the License.
+%%     http://www.apache.org/licenses/LICENSE-2.0
+%%
+%% Unless required by applicable law or agreed to in writing, software
+%% distributed under the License is distributed on an "AS IS" BASIS,
+%% WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+%% See the License for the specific language governing permissions and
+%% limitations under the License.
 %%
 %% %CopyrightEnd%
 %%
@@ -24,36 +25,37 @@
 
 -module(diameter_transport_SUITE).
 
--export([suite/0,
-         all/0,
-         groups/0,
-         init_per_suite/1,
-         end_per_suite/1]).
+%% all tests, no common_test dependency
+-export([run/0]).
 
-%% testcases
--export([start/1,
-         tcp_accept/1,
-         tcp_connect/1,
-         sctp_accept/1,
-         sctp_connect/1,
-         reconnect/1, reconnect/0,
-         stop/1]).
+%% common_test wrapping
+-export([
+         %% Framework functions
+         suite/0,
+         all/0,
+         init_per_suite/1,
+         end_per_suite/1,
+        
+         %% The test cases
+         parallel/1
+        ]).
 
 -export([accept/1,
          connect/1,
+         reconnect/1,
          init/2]).
 
 -include_lib("kernel/include/inet_sctp.hrl").
 -include("diameter.hrl").
 
--define(util, diameter_util).
+-include("diameter_util.hrl").
 
 %% Corresponding to diameter_* transport modules.
 -define(TRANSPORTS, [tcp, sctp]).
 
 %% Receive a message.
 -define(RECV(Pat, Ret), receive Pat -> Ret end).
--define(RECV(Pat), ?RECV(Pat, diameter_util:timestamp())).
+-define(RECV(Pat), ?RECV(Pat, diameter_lib:now())).
 
 %% Sockets are opened on the loopback address.
 -define(ADDR, {127,0,0,1}).
@@ -64,7 +66,7 @@
                                       = #diameter_caps{host_ip_address
                                                        = Addrs}}).
 
-%% The term we register after open a listening port with gen_tcp.
+%% The term we register after open a listening port with gen_{tcp,sctp}.
 -define(TEST_LISTENER(Ref, PortNr),
         {?MODULE, listen, Ref, PortNr}).
 
@@ -82,92 +84,114 @@
 %% Messages from gen_sctp.
 -define(SCTP(Sock, Data), {sctp, Sock, _, _, Data}).
 
+
+-define(TL(F),    ?TL(F, [])).
+-define(TL(F, A), ?LOG("DTRANSPS", F, A)).
+
+
 %% ===========================================================================
+%% common_test wrapping
 
 suite() ->
-    [{timetrap, {minutes, 2}}].
+    [{timetrap, {seconds, 270}}].
 
 all() ->
-    [start,
-     {group, all},
-     {group, all, [parallel]},
-     stop].
+    [parallel].
 
-groups() ->
-    [{all, [], tc()}].
-
-tc() ->
-    [tcp_accept,
-     tcp_connect,
-     sctp_accept,
-     sctp_connect,
-     reconnect].
 
 init_per_suite(Config) ->
-    [{sctp, ?util:have_sctp()} | Config].
+    ?DUTIL:init_per_suite(Config).
+
+end_per_suite(Config) ->
+    ?DUTIL:end_per_suite(Config).
+
+
+parallel(_) ->
+    ?TL("parallel -> entry"),
+    Res = run(),
+    ?TL("parallel -> done when"
+        "~n   Res: ~p", [Res]),
+    Res.
 
-end_per_suite(_Config) ->
-    ok.
 
 %% ===========================================================================
 
-start(_Config) ->
-    ok = diameter:start().
+%% run/0
+
+run() ->
+    ok = diameter:start(),
+    try
+        ?RUN([[fun run/1, {P,F}]
+              || P <- [sctp || ?HAVE_SCTP()] ++ [tcp],
+                 F <- [connect, accept, reconnect]])
+    after
+        diameter:stop()
+    end.
+
+%% run/1
+
+run({Prot, reconnect}) ->
+    ?TL("run(reconnect) -> entry with"
+        "~n   Prot: ~p", [Prot]),
+    Res = reconnect(Prot),
+    ?TL("run(reconnect) -> done when"
+        "~n   Res: ~p", [Res]),
+    Res;
+
+run({Prot, accept}) ->
+    ?TL("run(accept) -> entry with"
+        "~n   Prot: ~p", [Prot]),
+    Res = accept(Prot),
+    ?TL("run(accept) -> done when"
+        "~n   Res: ~p", [Res]),
+    Res;
+
+run({Prot, connect}) ->
+    ?TL("run(connect) -> entry with"
+        "~n   Prot: ~p", [Prot]),
+    Res = connect(Prot),
+    ?TL("run(connect) -> done when"
+        "~n   Res: ~p", [Res]),
+    Res.
 
-stop(_Config) ->
-    ok = diameter:stop().
 
 %% ===========================================================================
-%% tcp_accept/1
-%% sctp_accept/1
+%% accept/1
 %%
 %% diameter transport accepting, test code connecting.
 
-tcp_accept(_) ->
-    accept(tcp).
-
-sctp_accept(Config) ->
-    case lists:member({sctp, true}, Config) of
-        true  -> accept(sctp);
-        false -> {skip, no_sctp}
-    end.
-
-%% Start multiple accepting transport processes that are connected to
-%% with an equal number of connecting processes using gen_tcp/sctp
-%% directly.
-
--define(PEER_COUNT, 8).
-
 accept(Prot) ->
-    T = {Prot, make_ref()},
-    [] = ?util:run(?util:scramble(acc(2*?PEER_COUNT, T, []))).
+    ?TL("accept -> entry with"
+        "~n   Prot: ~p", [Prot]),
+
+    Ref = make_ref(),
+    true = diameter_reg:add_new({diameter_config, transport, Ref}), %% fake it
+    T = {Prot, Ref},
+    Res = ?RUN([{{?MODULE, [init, X, T]}, 15000}
+                || X <- [accept, gen_connect]]),
+
+    ?TL("accept -> done when"
+        "~n   Res: ~p", [Res]),
+    ok.
 
-acc(0, _, Acc) ->
-    Acc;
-acc(N, T, Acc) ->
-    acc(N-1, T, [{?MODULE, [init,
-                            element(1 + N rem 2, {accept, gen_connect}),
-                            T]}
-                 | Acc]).
 
 %% ===========================================================================
-%% tcp_connect/1
-%% sctp_connect/1
+%% connect/1
 %%
 %% Test code accepting, diameter transport connecting.
 
-tcp_connect(_) ->
-    connect(tcp).
-
-sctp_connect(Config) ->
-    case lists:member({sctp, true}, Config) of
-        true  -> connect(sctp);
-        false -> {skip, no_sctp}
-    end.
-
 connect(Prot) ->
+    ?TL("connect -> entry with"
+        "~n   Prot: ~p", [Prot]),
+
     T = {Prot, make_ref()},
-    [] = ?util:run([{?MODULE, [init, X, T]} || X <- [gen_accept, connect]]).
+    Res = ?RUN([{{?MODULE, [init, X, T]}, 15000}
+                 || X <- [gen_accept, connect]]),
+
+    ?TL("connect -> done when"
+        "~n   Res: ~p", [Res]),
+    ok.
+
 
 %% ===========================================================================
 %% reconnect/1
@@ -176,53 +200,87 @@ connect(Prot) ->
 %% doesn't try to establish a new connection until the old one is
 %% broken.
 
-reconnect() ->
-    [{timetrap, {minutes, 4}}].
-
 reconnect({listen, Ref}) ->
+    ?TL("reconnect(listen) -> entry with"
+        "~n   Ref: ~p", [Ref]),
     SvcName = make_ref(),
+    ?TL("reconnect(listen) -> start service (~p)", [SvcName]),
     ok = start_service(SvcName),
-    LRef = ?util:listen(SvcName, tcp, [{watchdog_timer, 6000}]),
+    ?TL("reconnect(listen) -> connect"),
+    LRef = ?LISTEN(SvcName, tcp, [{watchdog_timer, 6000}]),
+    ?TL("reconnect(listen) -> wait"),
     [_] = diameter_reg:wait({diameter_tcp, listener, {LRef, '_'}}),
+    ?TL("reconnect(listen) -> register new transport"),
     true = diameter_reg:add_new({?MODULE, Ref, LRef}),
 
     %% Wait for partner to request transport death.
+    ?TL("reconnect(listen) -> abort: await (request for) transport death"),
     TPid = abort(SvcName, LRef, Ref),
 
     %% Kill transport to force the peer to reconnect.
+    ?TL("reconnect(listen) -> kill transport"),
     exit(TPid, kill),
 
     %% Wait for the partner again.
-    abort(SvcName, LRef, Ref);
+    ?TL("reconnect(listen) -> abort: wait for partner again"),
+    Res = abort(SvcName, LRef, Ref),
+
+    ?TL("reconnect(listen) -> done when"
+        "~n   Res: ~p", [Res]),
+    ok;
 
 reconnect({connect, Ref}) ->
+    ?TL("reconnect(connect) -> entry with"
+        "~n   Ref: ~p", [Ref]),
+
     SvcName = make_ref(),
+    ?TL("reconnect(connect) -> subscribe"),
     true = diameter:subscribe(SvcName),
+    ?TL("reconnect(connect) -> start service (~p)", [SvcName]),
     ok = start_service(SvcName),
+    ?TL("reconnect(connect) -> wait"),
     [{{_, _, LRef}, Pid}] = diameter_reg:wait({?MODULE, Ref, '_'}),
-    CRef = ?util:connect(SvcName, tcp, LRef, [{connect_timer, 2000},
-                                              {watchdog_timer, 6000}]),
+    ?TL("reconnect(connect) -> connect"),
+    CRef = ?CONNECT(SvcName, tcp, LRef, [{connect_timer, 2000},
+                                         {watchdog_timer, 6000}]),
 
     %% Tell partner to kill transport after seeing that there are no
     %% reconnection attempts.
+    ?TL("reconnect(connect) -> abort (kill transport)"),
     abort(SvcName, Pid, Ref),
 
     %% Transport goes down and is reestablished.
+    ?TL("reconnect(connect) -> await transport down"),
     ?RECV(#diameter_event{service = SvcName, info = {down, CRef, _, _}}),
+    ?TL("reconnect(connect) -> await transport reconnect"),
     ?RECV(#diameter_event{service = SvcName, info = {reconnect, CRef, _}}),
+    ?TL("reconnect(connect) -> await transport up"),
     ?RECV(#diameter_event{service = SvcName, info = {up, CRef, _, _, _}}),
 
     %% Kill again.
+    ?TL("reconnect(connect) -> abort (kill transport again)"),
     abort(SvcName, Pid, Ref),
 
     %% Wait for partner to die.
+    ?TL("reconnect(connect) -> await partner death"),
     MRef = erlang:monitor(process, Pid),
-    ?RECV({'DOWN', MRef, process, _, _});
+    Res = ?RECV({'DOWN', MRef, process, _, _}),
 
-reconnect(_) ->
+    ?TL("reconnect(connect) -> done when"
+        "~n   Res: ~p", [Res]),
+    ok;
+
+reconnect(Prot) ->
+    ?TL("reconnect -> entry with"
+        "~n   Prot: ~p", [Prot]),
     Ref = make_ref(),
-    [] = ?util:run([{?MODULE, [reconnect, {T, Ref}]}
-                    || T <- [listen, connect]]).
+    Res = ?RUN([{{?MODULE, [reconnect, {T, Ref}]}, 240000}
+                || Prot == tcp,  %% ignore sctp
+                   T <- [listen, connect]]),
+
+    ?TL("reconnect -> done when"
+        "~n   Res: ~p", [Res]),
+    ok.
 
 start_service(SvcName) ->
     OH = diameter_util:unique_string(),
@@ -260,6 +318,10 @@ abort(SvcName, LRef, Ref)
 %% init/2
 
 init(accept, {Prot, Ref}) ->
+    ?TL("init(accept) -> entry with"
+        "~n   Prot: ~p"
+        "~n   Ref:  ~p", [Prot, Ref]),
+
     %% Start an accepting transport and receive notification of a
     %% connection.
     TPid = start_accept(Prot, Ref),
@@ -273,30 +335,60 @@ init(accept, {Prot, Ref}) ->
     %% Expect the transport process to die as a result of the peer
     %% closing the connection.
     MRef = erlang:monitor(process, TPid),
-    ?RECV({'DOWN', MRef, process, _, _});
+    Res = ?RECV({'DOWN', MRef, process, _, _}),
+
+    ?TL("init(accept) -> done when"
+        "~n   Res: ~p", [Res]),
+    ok;
 
 init(gen_connect, {Prot, Ref}) ->
+    ?TL("init(gen_connect) -> entry with"
+        "~n   Prot: ~p"
+        "~n   Ref:  ~p", [Prot, Ref]),
+
     %% Lookup the peer's listening socket.
-    [PortNr] = ?util:lport(Prot, Ref),
+    [PortNr] = ?LPORT(Prot, Ref),
 
     %% Connect, send a message and receive it back.
     {ok, Sock} = gen_connect(Prot, PortNr),
     Bin = make_msg(),
     ok = gen_send(Prot, Sock, Bin),
-    Bin = gen_recv(Prot, Sock);
+    Bin = gen_recv(Prot, Sock),
+
+    ?TL("init(gen_connect) -> done"),
+    ok;
 
 init(gen_accept, {Prot, Ref}) ->
+    ?TL("init(gen_accept) -> entry with"
+        "~n   Prot: ~p"
+        "~n   Ref:  ~p", [Prot, Ref]),
+
     %% Open a listening socket and publish the port number.
     {ok, LSock} = gen_listen(Prot),
     {ok, PortNr} = inet:port(LSock),
     true = diameter_reg:add_new(?TEST_LISTENER(Ref, PortNr)),
 
-    %% Accept a connection, receive a message and send it back.
+    %% Accept a connection, receive a message send it back, and wait
+    %% for the peer to close the connection.
     {ok, Sock} = gen_accept(Prot, LSock),
     Bin = gen_recv(Prot, Sock),
-    ok = gen_send(Prot, Sock, Bin);
+    ok = gen_send(Prot, Sock, Bin),
+    _Res = receive
+               {tcp_closed, Sock} = T ->
+                   T;
+               ?SCTP(Sock, {_, #sctp_assoc_change{}}) = T ->
+                   T
+           end,
+
+    ?TL("init(gen_accept) -> done when"
+        "~n   Res: ~p", [_Res]),
+    ok;
 
 init(connect, {Prot, Ref}) ->
+    ?TL("init(connect) -> entry with"
+        "~n   Prot: ~p"
+        "~n   Ref:  ~p", [Prot, Ref]),
+
     %% Lookup the peer's listening socket.
     [{?TEST_LISTENER(_, PortNr), _}]
         = diameter_reg:wait(?TEST_LISTENER(Ref, '_')),
@@ -310,10 +402,8 @@ init(connect, {Prot, Ref}) ->
     TPid ! ?TMSG({send, Bin}),
     Bin = bin(Prot, ?RECV(?TMSG({recv, P}), P)),
 
-    %% Expect the transport process to die as a result of the peer
-    %% closing the connection.
-    MRef = erlang:monitor(process, TPid),
-    ?RECV({'DOWN', MRef, process, _, _}).
+    ?TL("init(connect) -> done"),
+    ok.
 
 bin(sctp, #diameter_packet{bin = Bin}) ->
     Bin;
@@ -333,51 +423,51 @@ make_msg() ->
     <<1:8, Len:24, Bin/binary>>.
 
 %% crypto:rand_bytes/1 isn't available on all platforms (since openssl
-%% isn't) so roll our own.
+%% isn't) so roll our own. Not particularly random, but less verbose
+%% in trace.
 rand_bytes(N) ->
-    random:seed(diameter_util:seed()),
-    rand_bytes(N, <<>>).
-
-rand_bytes(0, Bin) ->
-    Bin;
-rand_bytes(N, Bin) ->
-    Oct = random:uniform(256) - 1,
-    rand_bytes(N-1, <<Oct, Bin/binary>>).
+    Oct = rand:uniform(256) - 1,
+    binary:copy(<<Oct>>, N).
 
 %% ===========================================================================
 
 %% start_connect/3
 
 start_connect(Prot, PortNr, Ref) ->
-    {ok, TPid, [?ADDR]} = start_connect(Prot,
-                                        {connect, Ref},
-                                        ?SVC([]),
-                                        [{raddr, ?ADDR},
-                                         {rport, PortNr},
-                                         {ip, ?ADDR},
-                                         {port, 0}]),
-    ?RECV(?TMSG({TPid, connected, _})),
+    {ok, TPid} = start_connect(Prot,
+                               {connect, Ref},
+                               ?SVC([]),
+                               [{raddr, ?ADDR},
+                                {rport, PortNr},
+                                {ip, ?ADDR},
+                                {port, 0}]),
+    connected(Prot, TPid),
     TPid.
 
+connected(sctp, TPid) ->
+    ?RECV(?TMSG({TPid, connected, _}));
+connected(tcp, TPid) ->
+    ?RECV(?TMSG({TPid, connected, _, [?ADDR]})).
+
 start_connect(sctp, T, Svc, Opts) ->
-    diameter_sctp:start(T, Svc, [{sctp_initmsg, ?SCTP_INIT} | Opts]);
+    {ok, TPid, [?ADDR]}
+        = diameter_sctp:start(T, Svc, [{sctp_initmsg, ?SCTP_INIT} | Opts]),
+    {ok, TPid};
 start_connect(tcp, T, Svc, Opts) ->
     diameter_tcp:start(T, Svc, Opts).
 
 %% start_accept/2
 
 start_accept(Prot, Ref) ->
-    {Mod, Opts} = tmod(Prot),
-    {ok, TPid, [?ADDR]} = Mod:start({accept, Ref},
-                                    ?SVC([?ADDR]),
-                                    [{port, 0} | Opts]),
+    {ok, TPid, [?ADDR]}
+        = start_accept(Prot, {accept, Ref}, ?SVC([?ADDR]), [{port, 0}]),
     ?RECV(?TMSG({TPid, connected})),
     TPid.
 
-tmod(sctp) ->
-    {diameter_sctp, [{sctp_initmsg, ?SCTP_INIT}]};
-tmod(tcp) ->
-    {diameter_tcp, []}.
+start_accept(sctp, T, Svc, Opts) ->
+    diameter_sctp:start(T, Svc, [{sctp_initmsg, ?SCTP_INIT} | Opts]);
+start_accept(tcp, T, Svc, Opts) ->
+    diameter_tcp:start(T, Svc, Opts).
 
 %% ===========================================================================
 
@@ -401,12 +491,13 @@ gen_listen(tcp) ->
 %% gen_accept/2
 
 gen_accept(sctp, Sock) ->
-    Assoc = ?RECV(?SCTP(Sock, {_, #sctp_assoc_change{state = comm_up,
-                                                     outbound_streams = O,
-                                                     inbound_streams = I,
-                                                     assoc_id = A}}),
-                  {O, I, A}),
-    putr(assoc, Assoc),
+    #sctp_assoc_change{state = comm_up,
+                       outbound_streams = OS,
+                       inbound_streams = IS,
+                       assoc_id = Id}
+        = ?RECV(?SCTP(Sock, {_, #sctp_assoc_change{} = S}), S),
+
+    putr(assoc, {OS, IS, Id}),
     {ok, Sock};
 gen_accept(tcp, LSock) ->
     gen_tcp:accept(LSock).
@@ -415,19 +506,29 @@ gen_accept(tcp, LSock) ->
 
 gen_send(sctp, Sock, Bin) ->
     {OS, _IS, Id} = getr(assoc),
-    {_, _, Us} = diameter_util:timestamp(),
-    gen_sctp:send(Sock, Id, Us rem OS, Bin);
+    gen_sctp:send(Sock, Id, erlang:unique_integer([positive]) rem OS, Bin);
 gen_send(tcp, Sock, Bin) ->
     gen_tcp:send(Sock, Bin).
 
 %% gen_recv/2
 
 gen_recv(sctp, Sock) ->
-    {_OS, _IS, Id} = getr(assoc),
+    {OS, IS, Id} = getr(assoc),
     receive
-        ?SCTP(Sock, {[#sctp_sndrcvinfo{assoc_id = Id}], Bin})
+        ?SCTP(Sock, {[#sctp_sndrcvinfo{assoc_id = I} = INFO], Bin})
           when is_binary(Bin) ->
-            Bin
+            case {I, Id} of
+                {Id, _} -> % assert
+                    Bin;
+                _ ->
+                    ?TL("unexpected assoc id in received info msg:"
+                        "~n   Expected Assoc ID: ~p"
+                        "~n      OS: ~p"
+                        "~n      IS: ~p"
+                        "~n   Received Assoc ID: ~p"
+                        "~n   Info:              ~p", [Id, OS, IS, I, INFO]),
+                    ct:fail({unexpected_assoc_id, I, Id})
+            end
     end;
 gen_recv(tcp, Sock) ->
     tcp_recv(Sock, <<>>).
